
/*
 * This device is designed to send data to Yamaha THR10 to select up to 60 patches.
 * The original idea and sketch has been made by Mathis Rosenhauer in 2016 - https://github.com/MathisRosenhauer/THR_Footswitch/blob/master/THR_Footswitch.ino
 * Fundamental help from Uwe Brandt evolution (OLED display and Read Name of the Patches) - https://www.mehlbrandt.de/thr/thr_10.html
 * This version has been elaborated by GP (me) in 2022 and doesn't use the SC Card (after weeks of frustration!) to keep it as lean as possible.
 * Furthermore I have changed the 2 buttons to a rotating encoder: the THR10 is a lovely practice amp, I found much useful to access the stored patches
 * quickly with an encoder instead of pushing tents time the buttons! 
 * The push-button on the encoder is used to advance to the next 10th patches at the time: so you can organize your patches by kind (e.g. 1-9 "Classic Rock" 10-19 "Clean" 20-29 "Jazz"...)
 * 
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version 2
 *      of the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, see <http://www.gnu.org/licenses/>
*/

/*             from Mathis Rosenhauer original sketch:
 *              
 *             withour SD the file patches.h has to be included
               which needs to define a PROGMEM variable named patches. 
               Use patchdump.py to generate patches.h from a .YDL file. E.g.
               ./patchdump.py yamaha.ydl -n 25 > patches.h
               to dump the first 25 patches from yamaha.ydl. The Arduino IDE will
               complain if the patches won't fit into PROGMEM. On a Uno -n 100
               (all patches) are too many but ~75 worked at one point in time.
    

   Practically to generate the file patches.h first produce a script (use IDLE in Launchpad) called patchdumpt.py (reported here for your convenience and file attached):

   ------------------------------------------------------------------------------------------------------
   
            #!/usr/bin/env python
            import argparse

            parser = argparse.ArgumentParser()
            parser.add_argument("patchfile", help="name of patch file in YDL format")
            parser.add_argument("-n", "--numpatches", type=int, default=100,
                    help="number of patches to convert")
            args = parser.parse_args()

            with open(args.patchfile, 'rb') as f:
                patches = f.read(13 + 261 * min(args.numpatches, 100))

            if patches:
                 print("const uint8_t patches[] PROGMEM = {")
                  for b in patches:
                    print("0x%02x, " % ord(b))
                  print("};")
   -------------------------------------------------------------------------------------------------------
   
   Then use the Terminal (Mac) and type the command:

   .....$ python ./patchdump.py yamaha.ydl -n 60 > patches.h

   (60 is the total number of patches: sketch uses 98% of program storage space, practically this is the MAX for Arduino UNO capacity!)
   Be sure that the Preset file ("yamaha.ydl" from THR10 Editor) and the script patchdump.py 
   are in the root directory (user\..\) when the file patches.h has been generated, 
   copy it in the same folder of the Arduino .ini sketch and add the file as well in the Sketch\Add File...

   one day I'll try to:
    - add the "Compressor" button to ON/OFF it directly
    - use Arduino nano instead of UNO
    - integrate the device in the Remote Foot Switches for THR10! https://www.instructables.com/Yamaha-THR10-Remote-Footswitch-Controller-With-OLE/
   
*/

#include <SPI.h>                   // OLED library
#include <SSD1306Ascii.h>          // OLED library
#include <SSD1306AsciiAvrI2c.h>    // OLED library
#define I2C_ADDRESS 0x3C           // OLED address

#include <avr/pgmspace.h>
#include <Usb.h>
#include <usbh_midi.h>

#include "patches.h"

// Satisfy IDE, which only needs to see the include statment in the ino.
#ifdef dobogusinclude
#include <spi4teensy3.h>
#include <SPI.h>
#endif


// ----------------------- encoder --------------------------
const uint8_t button_l_pin = 2;     // encoder A
const uint8_t button_r_pin = 3;     // encoder B
const uint8_t encoder_button = 4;   // encoder push-button
int encoder_state = 0;
volatile byte pinALast = HIGH;
volatile byte stateA = HIGH;
// ----------------------- encoder --------------------------


uint8_t patch_id = 0;
String stringName;
int i;
byte patchName[13] = {'Y','A','M','A','H','A',' ','T','H','R','-','1','0'};

USB Usb;
USBH_MIDI Midi(&Usb);

SSD1306AsciiAvrI2c oled;


void setup()
{
  Serial.begin(9600);

    stringName = String("");
    pinMode(button_r_pin, INPUT_PULLUP);       // Right (clockwise) encoder increases patch id.
    pinMode(button_l_pin, INPUT_PULLUP);       // Left (counter-clockwise) encoder decreases patch id.
    pinMode(encoder_button, INPUT_PULLUP);    // Puch-Button encoder will be used to increase patches by 10

// --------------------------------------------------------------- encoder -----------------------------------------------------------------------------
attachInterrupt(digitalPinToInterrupt(button_r_pin),PinB,RISING); // set an interrupt on PinB, looking for a RISING edge signal and executing the "PinB" Interrupt Service Routine (below)  
attachInterrupt(digitalPinToInterrupt(button_l_pin),PinA,FALLING); // set an interrupt on PinA, looking for a FALLING edge signal and executing the "PinA" Interrupt Service Routine (below)
// --------------------------------------------------------------- encoder -----------------------------------------------------------------------------


// --------------------------------   USBH setup   --------------------------------
    if (Usb.Init() == -1) {
        Serial.println(F("USB host init failed."));
        while(1); //halt
    }
    delay(200);
    Serial.println(F("USB initialized."));


 oled.begin(&Adafruit128x64, I2C_ADDRESS);
 oled.clear(); 

// Welcome message to dispaly at start
 
     for (i = 0; i < 13; i++) {     
    stringName = stringName + char(patchName[i]);                         
     Serial.println(stringName);    
    }
    text();

}


// ----------------------- encoder --------------------------

      void PinA(){
        cli(); //stop interrupts happening before we read pin values
        stateA = digitalRead(button_l_pin);
 
       if ((pinALast == HIGH) && (stateA == LOW)) {
          if (digitalRead(button_r_pin) == HIGH) {   
      encoder_state = -1;
           } else {
      encoder_state = 1;
                   }
                                                  }
       pinALast = stateA;
       sei(); //restart interrupts
            }

       void PinB(){
         cli(); //stop interrupts happening before we read pin values
         stateA = digitalRead(button_l_pin);
  
          if ((pinALast == HIGH) && (stateA == LOW)) {
             if (digitalRead(button_r_pin) == HIGH) {  
        encoder_state = -1;
              } else {
        encoder_state = 1;
           }
                                             }
        pinALast = stateA;
        sei(); //restart interrupts
            }


// --------------------------------- Send Patches to THR10 by Mathis Rosenhauer  ------------------------------
void send_patch(uint8_t patch_id)
{

     Serial.println(encoder_state);
     Serial.println(stringName);
     Serial.println(patch_id);
  
    uint8_t prefix[] = {
        0xf0, 0x43, 0x7d, 0x00, 0x02, 0x0c, 0x44, 0x54,
        0x41, 0x31, 0x41, 0x6c, 0x6c, 0x50, 0x00, 0x00,
        0x7f, 0x7f
    };
    uint8_t cs = 0x71; // prefix checksum
    const size_t patch_size = 0x100;
    size_t offset = 13 + (patch_id - 1) * (patch_size + 5);
    uint8_t chunk[16];
    uint8_t suffix[2];

        Midi.SendSysEx(prefix, sizeof(prefix));
        
        for (size_t j = 0; j < patch_size; j += sizeof(chunk)) {
            for (size_t i = 0; i < sizeof(chunk); i++) {
                
                if (j + i == patch_size - 1) {
                    // Last byte of patch has to be zero.
                    chunk[i] = 0;            } 
                    
                else { chunk[i] = pgm_read_byte(patches + offset++); }
                
                cs += chunk[i];
                                                        }
                                                        
            Midi.SendSysEx(chunk, sizeof(chunk));
                                                                }
        // Calculate check sum.
        suffix[0] = (~cs + 1) & 0x7f;
        // end SysEx
        suffix[1] = 0xf7;

            Midi.SendSysEx(suffix, sizeof(suffix));


}

// ------------------------------------------------------------------------- Read Patches name by Uwe Brandt  ---------

void read_name(uint8_t patch_id){
 
 const size_t patch_size = 0x100;
 int i;
 stringName = String("");
 size_t offset = 13 + (patch_id - 1) * (patch_size + 5);
  
     for (i = 0; i < 13; i++) {

      patchName[i] = pgm_read_byte(patches + offset++);
   
     stringName = stringName + char(patchName[i]);
             
    }
}

void text() {

//   Serial.println(stringName);
//   Serial.println(patch_id);
         
      oled.clear(); 
 oled.setCursor(13, 0);
     oled.setFont(ZevvPeep8x16);  
     oled.set1X();
     oled.println(stringName);
     oled.println();
     delay(10);  
 
   byte displayPointer = 50;     
       if (patch_id == 1 ) displayPointer = 45;
       if (patch_id > 9 ) displayPointer = 30;
       if (patch_id > 19) displayPointer = 40;
 oled.setCursor(displayPointer,2);
     oled.setFont(lcdnums14x24);  
     oled.set2X();
     oled.println(patch_id);
     oled.println();
     delay(20);

}



void loop() {
 
 // -------------------------------- encoder ---------------------------------
         stateA = digitalRead(button_l_pin);
       if ((pinALast == HIGH) && (stateA == LOW)) {
          if (digitalRead(button_r_pin) == HIGH) {   
      encoder_state = -1;
           } else {
      encoder_state = 1;
                   }
                                                  }
       pinALast = stateA;
 // -------------------------------- encoder ---------------------------------
      
 
    static bool thr_connected = false;

    const int npatches = (sizeof(patches) - 13) / 261;

    Usb.Task();
    if (Usb.getUsbTaskState() == USB_STATE_RUNNING) {
        if (thr_connected) {

 //    Serial.println(encoder_state);

            if (digitalRead(encoder_button) == LOW) { 
                patch_id = constrain(patch_id + 10, 1, npatches);  delay(500);   
                patch_id = (patch_id / 10) * 10;
                if (patch_id == npatches) {patch_id = 1;}                      
                
                read_name(patch_id);          
                send_patch(patch_id);
                text();
                                                    } 
            
            if (encoder_state != 0) {
                patch_id = constrain(patch_id + encoder_state, 1, npatches);
                read_name(patch_id);          
                send_patch(patch_id);
                text();
                
                encoder_state = 0;             
                                    }
            
        } else {   // A new device was connected. Assume it is a THR10. Just don't connect anything else.
            delay(1000);
            thr_connected = true;


                
      //     send_patch(patch_id);
       //     read_name(patch_id);
       //     text();  
               
        }
    } else if (thr_connected) {
        thr_connected = false;
        stringName = String("WAITING");
        text();
    }

}
